//////////////////////
// CControls.h

#ifndef __CCONTROLS_H__
#define __CCONTROLS_H__

#include "CFont.h"
#include "CModeScreen.h"

class CDisplay;
class CInput;
class COptions;
class CTimer;
class CSound;
class CMosaic;

enum EGameMode;

//******************************************************************************************************************************
//******************************************************************************************************************************
//******************************************************************************************************************************

//! The controls screen that appears when choosing Options in the main menu.

class CControls : public CModeScreen
{
private:

    float           m_ModeTime;                 //!< Time (in seconds) that elapsed since the mode has started
    bool            m_HaveToExit;               //!< Do we have to exit this mode?
    float           m_ExitModeTime;             //!< Mode time when we realized we have to exit (used for blackscreen)
    CFont           m_Font;                     //!< Font object used to draw strings
    int             m_Cursor;                   //!< Number of the menu item the cursor hand is pointing to
    int             m_PlayerInput;              //!< Number of the currently selected player input (keyboard X, joystick X...)
    bool            m_WaitingForInput;          //!< Are we waiting for the user to activate the control he wants to reconfigure?
    float           m_WaitingForInputAfter;     //!< Time left in seconds to wait before starting to wait for the user to activate the control he wants to reconfigure.
    bool            m_SongStarted;              //!< Did we start playing the song after the black screen?
    CMosaic*        m_pMosaic;

public:

                    CControls();                        //!< Constructor. Initialize some members.
    virtual         ~CControls();                       //!< Destructor. Does nothing.
    inline void     SetDisplay (CDisplay *pDisplay);    //!< Set link to the display object to use
    void            Create (void);                      //!< Initialize the object
    void            Destroy (void);                     //!< Uninitialize the object
    void            OpenInput (void);                   //!< Get access to the input this object needs
    void            CloseInput (void);                  //!< Release access to the input this object needs
    EGameMode       Update (void);                      //!< Update the object and return what game mode should be set
    void            Display (void);                     //!< Display the game screen
};

//******************************************************************************************************************************
//******************************************************************************************************************************
//******************************************************************************************************************************

inline void CControls::SetDisplay (CDisplay *pDisplay)
{
    CModeScreen::SetDisplay (pDisplay);
    m_Font.SetDisplay (m_pDisplay);
}

//******************************************************************************************************************************
//******************************************************************************************************************************
//******************************************************************************************************************************

#endif  // __CCONTROLS_H__
