//////////////////////
// CMatch.h

#ifndef __CMATCH_H__
#define __CMATCH_H__

enum EGameMode;

class CDisplay;
class CInput;
class COptions;
class CScores;
class CTimer;
class CSound;
class CPauseMessage;
class CHurryMessage;

enum ESong;

#include "CBoard.h"
#include "CArena.h"
#include "CClock.h"
#include "CAiManager.h"
#include "CModeScreen.h"

//******************************************************************************************************************************
//******************************************************************************************************************************
//******************************************************************************************************************************

#define NO_WINNER_PLAYER    -1      //<! Value for a winner player number if there is no winner

//******************************************************************************************************************************
//******************************************************************************************************************************
//******************************************************************************************************************************

//! The match screen, managing the arena and the board.

class CMatch : public CModeScreen
{
private:

    CBoard          m_Board;                    //!< Board object
    CClock          m_Clock;                    //!< Clock object
    CArena          m_Arena;                    //!< Arena object
    CAiManager      m_AiManager;                //!< Computer brain
    bool            m_NoComputer;               //!< True if no computer is playing in this match
    bool            m_MatchOver;                //!< Is match over? (ie. there is a result : winner or draw game)
    int             m_WinnerPlayer;             //!< Number of the player who won if there is a winner
    ESong           m_CurrentSong;              //!< Current song being played
    bool            m_IsSongPlaying;            //!< Is the match song playing?
    bool            m_NoticedTimeUp;            //!< Did we notice that time is up and do what is necessary?
    CPauseMessage*  m_pPauseMessage;            //!< Pause message object, instanciated when the match is paused
    CHurryMessage*  m_pHurryMessage;            //!< Hurry up message object, instanciated when the arena starts to close
    float           m_ModeTime;                 //!< Time (in seconds) that elapsed since the mode has started
    float           m_ExitModeTime;             //!< Mode time when we have to start the last black screen
    EGameMode       m_ExitGameMode;             //!< Game mode to ask for when exiting
    bool            m_HaveToExit;               //!< Do we have to exit this mode?
    
    void            CreateMainComponents (void);
    void            DestroyHurryUpMessage (void);
    void            DestroyPauseMessage (void);
    void            DestroyMainComponents (void);
    void            PlaySong (void);
    void            StopSong (void);
    void            ProcessPlayerCommands (void);
    void            UpdateMatch (void);
    void            ManagePauseMessage (void);
    void            ManageHurryUpMessage (void);
    void            ManageMatchOver (void);
    void            DisplayMatchScreen (void);
    void            DisplayHurryUpMessage (void);
    void            DisplayPauseMessage (void);
    
public:

                    CMatch();                           //!< Constructor. Initialize some members.
    virtual         ~CMatch();                          //!< Destructor. Does nothing.
    inline void     SetDisplay (CDisplay *pDisplay);    //!< Set link to the display object to use
    inline void     SetOptions (COptions *pOptions);    //!< Set link to the options object to use
    inline void     SetScores (CScores *pScores);       //!< Set link to the scores object to use
    inline void     SetTimer (CTimer *pTimer);          //!< Set link to the timer object to use
    inline void     SetSound (CSound *pSound);          //!< Set link to the sound object to use
    void            Create (void);                      //!< Initialize the object
    void            Destroy (void);                     //!< Uninitialize the object
    void            OpenInput (void);                   //!< Get access to the input this object needs
    void            CloseInput (void);                  //!< Release access to the input this object needs
    inline int      GetWinnerPlayer (void);             //!< Get the number of the player who won this match
    EGameMode       Update (void);                      //!< Update the object and return what game mode should be set
    void            Display (void);                     //!< Display on the screen
};

//******************************************************************************************************************************
//******************************************************************************************************************************
//******************************************************************************************************************************

inline void CMatch::SetDisplay (CDisplay *pDisplay)
{
    CModeScreen::SetDisplay (pDisplay);

    m_Board.SetDisplay (pDisplay);
    m_Arena.SetDisplay (pDisplay);
}

inline void CMatch::SetOptions (COptions *pOptions)
{
    CModeScreen::SetOptions (pOptions);

    m_Board.SetOptions (pOptions);
    m_Arena.SetOptions (pOptions);
}

inline void CMatch::SetScores (CScores *pScores)
{
    m_Board.SetScores (pScores);
}

inline void CMatch::SetTimer (CTimer *pTimer)
{
    CModeScreen::SetTimer (pTimer);    

    m_Board.SetTimer (pTimer);
} 

inline void CMatch::SetSound (CSound *pSound)
{
    CModeScreen::SetSound (pSound);    

    m_Arena.SetSound (pSound);
}

//******************************************************************************************************************************
//******************************************************************************************************************************
//******************************************************************************************************************************

inline int CMatch::GetWinnerPlayer (void)
{
    return m_WinnerPlayer;
}

//******************************************************************************************************************************
//******************************************************************************************************************************
//******************************************************************************************************************************

#endif  // __CMATCH_H__
