/////////////////////////
// CSound.h

#ifndef __CSOUND_H__
#define __CSOUND_H__

#include <fmod.h>

//******************************************************************************************************************************
//******************************************************************************************************************************
//******************************************************************************************************************************

enum ESample
{
    SAMPLE_BOMB_DROP = 0,
    SAMPLE_BOMBER_DEATH,
    SAMPLE_BOMB_BOUNCE,
    SAMPLE_BOMBER_PUNCH,
    SAMPLE_BOMBER_THROW,
    SAMPLE_BOMBER_LOSE_ITEM,
    SAMPLE_BREAK_1,
    SAMPLE_BREAK_2,
    SAMPLE_DRAW_GAME,
    SAMPLE_EXPLOSION_01_1,
    SAMPLE_EXPLOSION_01_2,
    SAMPLE_EXPLOSION_02_1,
    SAMPLE_EXPLOSION_02_2,
    SAMPLE_EXPLOSION_03_1,
    SAMPLE_EXPLOSION_03_2,
    SAMPLE_EXPLOSION_04_1,
    SAMPLE_EXPLOSION_04_2,
    SAMPLE_EXPLOSION_05_1,
    SAMPLE_EXPLOSION_05_2,
    SAMPLE_EXPLOSION_06_1,
    SAMPLE_EXPLOSION_06_2,
    SAMPLE_EXPLOSION_07_1,
    SAMPLE_EXPLOSION_07_2,
    SAMPLE_EXPLOSION_08_1,
    SAMPLE_EXPLOSION_08_2,
    SAMPLE_EXPLOSION_09_1,
    SAMPLE_EXPLOSION_09_2,
    SAMPLE_EXPLOSION_10_1,
    SAMPLE_EXPLOSION_10_2,
    SAMPLE_HURRY,
    SAMPLE_ITEM_FUMES,
    SAMPLE_MENU_NEXT,
    SAMPLE_MENU_PREVIOUS,
    SAMPLE_MENU_BEEP,
    SAMPLE_MENU_ERROR,
    SAMPLE_PAUSE,
    SAMPLE_PICK_ITEM_1,
    SAMPLE_PICK_ITEM_2,
    SAMPLE_RING_DING,
    SAMPLE_SICK_1,
    SAMPLE_SICK_2,
    SAMPLE_SICK_3,
    SAMPLE_VICTORY,
    SAMPLE_WALL_CLAP_1,
    SAMPLE_WALL_CLAP_2,
    SAMPLE_WINNER,
    NUM_SAMPLES
};

enum ESong
{
    SONG_NONE = -1,
    SONG_MATCH_MUSIC_1_NORMAL = 0,
    SONG_MATCH_MUSIC_1_FAST,
    SONG_MENU_MUSIC,
    SONG_CONTROLS_MUSIC,
    SONG_TITLE_MUSIC,
    NUM_SONGS
};

//******************************************************************************************************************************
//******************************************************************************************************************************
//******************************************************************************************************************************

class CSound
{
private:

    HMODULE m_hModule;                          //!< Connection to the resources
    bool m_GlobalPause;                         //!< Is the sound paused?
    bool m_SoundOK;                             //!< Could FMOD be initialized? This may be false if there is no sound card
    FSOUND_SAMPLE* m_Samples [NUM_SAMPLES];     //!< The available samples
    FMUSIC_MODULE* m_Songs [NUM_SONGS];         //!< The available songs

    bool GetSoundResource (int ResourceID, LPVOID &pData, DWORD &DataSize);
    bool LoadSample (ESample Sample, int ResourceID);
    void FreeSample (ESample Sample);
    bool LoadSong (ESong Song, int ResourceID);
    void FreeSong (ESong Song);

public:

    CSound (void);
    ~CSound (void);

    inline void SetModuleHandle (HMODULE hModule);             //!< Set the connection to the resources
    bool        Create (void);                                 //!< Initialize the object
    void        Destroy (void);                                //!< Uninitialize the object
    void        SetPause (bool Pause);                         //!< Pause or resume the sound
    inline bool IsPaused (void);
    void        PlaySong (ESong Song);                         //!< Start playing a song
    void        StopSong (ESong Song);                         //!< Stop playing a specific song
    void        PlaySample (ESample Sample);                   //!< Start playing a sample
    void        SetSampleVolume (int VolumePerCent);           //!< Set the volume for all samples
    void        SetSongVolume (ESong Song, int VolumePerCent); //!< Set the volume for a specific song
};

//******************************************************************************************************************************
//******************************************************************************************************************************
//******************************************************************************************************************************

inline void CSound::SetModuleHandle (HMODULE hModule)
{
    m_hModule = hModule;
}

inline bool CSound::IsPaused (void)
{
    return m_GlobalPause;
}

//******************************************************************************************************************************
//******************************************************************************************************************************
//******************************************************************************************************************************

#endif // __CSOUND_H__