//////////////////////
// CWinner.h

#ifndef __CWINNER_H__
#define __CWINNER_H__

class CDisplay;
class CInput;
class COptions;
class CTimer;
class CScores;
class CMatch;
class CSound;
class CMosaic;

enum EGameMode;

#include "CModeScreen.h"

//******************************************************************************************************************************
//******************************************************************************************************************************
//******************************************************************************************************************************

//! The winner (scoreboard) screen that appears after a player has won one battle.

class CWinner : public CModeScreen
{
private:

    CScores*        m_pScores;                  //!< Link to the scores object to use
    CMatch*         m_pMatch;                   //!< Link to the match object to use
    float           m_LightsTimer;              //!< Lights animation timer
    float           m_HappyBomberTimer;         //!< Happy bomber animation timer
    float           m_SadBomberTimer;           //!< Sad bomber animation timer
    int             m_LightSpriteOffset;        //!< Sprite offset to use to switch light colors
    int             m_HappyBomberSpriteOffset;  //!< Sprite offset to use to have a happy bomber
    int             m_SadBomberSpriteOffset;    //!< Sprite offset to use to have a sad bomber
    bool            m_PlayedSound;              //!< Did we start playing the victory sound?
    CMosaic*        m_pMosaic;
    float           m_ModeTime;                 //!< Time (in seconds) that elapsed since the mode has started
    float           m_ExitModeTime;             //!< Mode time when we have to start the last black screen
    EGameMode       m_ExitGameMode;             //!< Game mode to ask for when exiting
    bool            m_HaveToExit;               //!< Do we have to exit this mode?
    
public:

                    CWinner();                          //!< Constructor. Initialize some members.
    virtual         ~CWinner();                         //!< Destructor. Does nothing.
    inline void     SetScores (CScores *pScores);       //!< Set link to the scores object to use
    inline void     SetMatch (CMatch *pMatch);          //!< Set link to the match object to use
    void            Create (void);                      //!< Initialize the object
    void            Destroy (void);                     //!< Uninitialize the object
    void            OpenInput (void);                   //!< Get access to the input this object needs
    void            CloseInput (void);                  //!< Release access to the input this object needs
    EGameMode       Update (void);                      //!< Update the object and return what game mode should be set
    void            Display (void);                     //!< Display on the screen
};

//******************************************************************************************************************************
//******************************************************************************************************************************
//******************************************************************************************************************************

inline void CWinner::SetScores (CScores *pScores)
{
    m_pScores = pScores;
}

inline void CWinner::SetMatch (CMatch *pMatch)
{
    m_pMatch = pMatch;
}

//******************************************************************************************************************************
//******************************************************************************************************************************
//******************************************************************************************************************************

#endif  // __CWINNER_H__
